/*
 * Parser for the burgomat grammar
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "lexer.h"
#include "parser.h"

static struct item_info *parser_attribinfo(struct menu_info *menu, 
										   struct lexer *lex)
{
	struct token *tok;
	struct item_info *info = menu->attr;

	tok = lexer_gettoken(lex);

	// Is everything as it should be?
	if (!tok || tok->type != TOKEN_STRING) {
		printf("Parse error row %d near col %d\n", lex->row, lex->col);
		return 0;
	}

	// Apparently it is. Find the mf'ing attrib.
	while(info->name) {
		if (!strcmp(tok->string, info->id)) return info;
		info++;
	}
	printf("No such modifier '%s'\n", tok->string);
	return 0;
}

static struct attrib *parser_modif(struct menu_info *menu, struct lexer *lex)
{
	struct attrib *top = 0, *atmp;
	struct item_info *info;
	struct token *tok;

	// Loop for attributes.
	for(;;) {
		tok = lexer_gettoken(lex);

		//End? This is wrong
		if(!tok) {
			printf("Unexpected end of tokens at row %d, col %d\n",
				   lex->row, lex->col);
			return (struct attrib *)-1;
		}
		// End of it.
		if (tok->type == TOKEN_SPECIAL && tok->val == ';') return top;
		
		// There are attribs!
		if (tok->type == TOKEN_QUANT) {
			info = parser_attribinfo(menu, lex);

			// Allocate, and initialize;
			if (!(atmp = (struct attrib *)malloc(sizeof(struct attrib))) 
				|| !info) return (struct attrib *)-1;
			atmp->next = top;
			atmp->quant = tok->val;
			atmp->info = info;
			top = atmp;
		} else {
			// Someone messed up.
			printf("Unexpected token on row %d before col %d\n",
				   lex->row, lex->col);
			return (struct attrib *)-1;
		}		
	}
}

static struct item *parser_stmt(struct menu_info *menu, struct token *tok, 
								struct lexer *lex)
{
    struct item *item;
    struct item_info *inf;

    // Initialize item, motherfucker.
    if (!(item = (struct item *)malloc(sizeof(struct item)))) return 0;
    item->next = 0;
    item->quant = 1;
    item->attr = 0;
    item->info = 0;

    // <stmt>  ::= <number> <prod> <modif> ; ?
    if (tok->type == TOKEN_NUMBER) {
		item->quant = tok->val;
		lexer_freetoken(tok);
		tok = lexer_gettoken(lex);
    }

    // <prod> <modif> ;
    if(!tok || tok->type != TOKEN_STRING) {
		free(item);
		printf("Parse error near row %d, col %d\n", lex->row, lex->col);
		return 0;
    }
    
    // Find the item in question.
    inf = menu->item;
    while (inf->name) {
		if (!strcmp(inf->id, tok->string))
			item->info = inf;
		inf++;
    }

    if (!item->info) {
		printf("Unknown product id '%s' at row %d, col %d\n", tok->string, 
			   lex->row, lex->col); 
		free(item);
		return 0;
    }

    // Parse modifiers. -1 on error, 0 if no modifiers.
    item->attr = parser_modif(menu, lex);
    if ((int)item->attr == -1) {
		free(item);
		return 0;
    } else return item;
}

static struct item *parser_parseitems(struct menu_info *menu, 
									  struct lexer *lex)
{
    struct item *top = 0, *itmp;
    struct token *tok;

    // Loop for items.
    // Check for start
    tok = lexer_gettoken(lex);
    if (!tok || tok->type != TOKEN_SPECIAL || tok->val != '{') {
		printf("Parse error row %d before col %d. Expected {\n",
			   lex->row, lex->col);
		return 0;
    }

    // Loop for items, if any present.
    for(;;) {
		tok = lexer_gettoken(lex);
		if (!tok) {
			printf("Missing } at row %d col %d\n", lex->row, lex->col);
			return 0;
		} else if (tok->type == TOKEN_SPECIAL) { 

			// Check for special token
			// Only } valid, wich ends loop
			if (tok->val == '}') { 
				lexer_freetoken(tok);
				return top;
			} else {
				printf("Unexpected %c at row %d, col %d\n", tok->val,
					   lex->row, lex->col);
				return 0;
			}
		
			// No end, something else.
		} else itmp = parser_stmt(menu, tok, lex);
		
		// Handle chaining of items
		if (itmp) {
			itmp->next = top;
			top = itmp;
		}
    }
}

static struct discount_info *parser_discount(struct menu_info *menu, 
											 struct lexer *lex)
{
    struct token *tok;
    struct discount_info *dtmp;

    // Check for the right type
    tok = lexer_gettoken(lex);
    if (!tok || tok->type != TOKEN_STRING) {
		printf("Parse error on row %d, before col %d\n",
			   lex->row, lex->col);
		return 0;
    }

    // Find discount type.
    dtmp = menu->disc;
    while (dtmp->name) {
		if (!(strcmp(tok->string, dtmp->id))) return dtmp;
		dtmp++;
    }
    printf("Illegal discount type %s\n", tok->string);
    return 0;
}

struct query *parser(struct menu_info *menu, struct lexer *lex)
{
    struct query *q;
    struct token *tok;

    // Initialize the query
    if (!(q = (struct query *)malloc(sizeof(struct query)))) return 0;
    q->items = 0;

    if (!(tok = lexer_gettoken(lex))) {
		printf("Parse error at row %d, col %d\n", lex->row, lex->col);
		free(q);
		return 0;
    }

    // What operation might this be?
    if (!strcasecmp("Order", tok->string))
		q->op = OP_ORDER;
    else if (!strcasecmp("Price", tok->string))
		q->op = OP_PRICE;
    else {
		printf("Illegal operation: %s\n", tok->string);
		lexer_freetoken(tok);
		free(q);
		return 0;
    }
    lexer_freetoken(tok);

    // Find the discount type.
    tok = lexer_gettoken(lex);
    if (!tok || tok->type != TOKEN_SPECIAL || tok->val != '(') {
		printf("Parse error on row %d before col %d. Expecting (\n",
			   lex->row, lex->col);
		if (tok) lexer_freetoken(tok);
		return 0;
    }
    lexer_freetoken(tok);

    if (!(q->disc = parser_discount(menu, lex))) return 0;

    tok = lexer_gettoken(lex);
    if (!tok || tok->type != TOKEN_SPECIAL || tok->val != ')') {
        printf("Parse error on row %d before col %d. Expecting )\n",
               lex->row, lex->col);
        if (tok) lexer_freetoken(tok);
        return 0;
    }
    lexer_freetoken(tok);

    // Parse the item list and insert into query.
    if (!(q->items = parser_parseitems(menu, lex))) return 0;

    // Damn, all worked well.
    return q;
}
