/*
 * Parser for the burgomat grammar
 */

#ifndef __BURGOPARSER_H__
#define __BURGOPARSER_H__

#include "lexer.h"

// Query operands
#define OP_ORDER 0
#define OP_PRICE 1

// Item flags
#define IFL_DISCOUNT 1

/*
 * Data structures used to define the things available at this
 * burgomat<tm>
 */
struct item_info {
    char *name;
    char *id;
    int price;
    int flags;
};

struct discount_info {
    char *name;
    char *id;
    float frac;
};

struct menu_info {
    struct discount_info *disc;
    struct item_info *item;
    struct item_info *attr;
};

/** Information about an attribute */
struct attrib {
	struct attrib *next;
	struct item_info *info;
    int quant;
};

/** An item of food, with info */
struct item {
    struct item *next;
    int quant;
    struct item_info *info;
    struct attrib *attr;
};

/** The complete query */
struct query {
    int op;
    struct discount_info *disc;
    struct item *items;    
};

/* Parser functions */
struct query *parser(struct menu_info *menu, struct lexer *lex);

#endif // __BURGOPARSER_H__
